/*
 * Created by SharpDevelop.
 * User: LYCJ
 * Date: 20/11/2006
 * Time: 21:59
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;

namespace Cake3
{
	/// <summary>
	/// Represent an archive handler, act as a layer between cakdir3 and archive dlls.
	/// </summary>
	public abstract class CakArchiver
	{
		internal bool canList = false;
		internal bool canAdd = false;
		internal bool canExtract = false;		
		internal bool canAlarmOverwrite = false;
		internal bool archiveOpened = false;	
		internal virtual string name { get { return "cak"; } }
		internal string archiveName;
		/// <summary>
		/// Occurs when the Listing an archive.
		/// </summary>
		public ItemListEventHandler OnItemList;
		/// <summary>
		/// Occurs when message received from inner wrapper.
		/// </summary>
		public MessageEventHandler OnMessage;
		/// <summary>
		/// Occurs when there is an error.
		/// </summary>
		public ErrorEventHandler OnError;
		/// <summary>
		/// Occurs when progress update.
		/// </summary>
		public ProgressEventHandler OnProgress;
		/// <summary>
		/// Occurs when confirm overwrite.
		/// </summary>
		public OverwriteEventHandler OnOverwrite;
		/// <summary>
		/// Occurs when request password.
		/// </summary>
		public PasswordEventHandler OnPassword;		
		/// <summary>
		/// Gets or sets the archivename.
		/// </summary>
		public string ArchiveName
		{
			get { return archiveName; }
			set { setArchiveName(value); }
		}
		
		internal virtual void setArchiveName(string anArchive)
		{
			archiveName = anArchive;
		}		
		/// <summary>
		/// Initializes a new instance of the CakArchiver class. 
		/// You shouldnt call this directly.
		/// </summary>
		public CakArchiver()
		{
							
		}
		
		internal void SetItemListEventHandler(ItemListEventHandler handler)
		{
			this.OnItemList = handler;
		}
		
		internal void SetMessageEventHandler(MessageEventHandler handler)
		{
			this.OnMessage = handler;
		}
		
		internal void SetErrorEventHandler(ErrorEventHandler handler)
		{
			this.OnError = handler;
		}
		
		internal void SetProgressEventHandler(ProgressEventHandler handler)
		{
			this.OnProgress = handler;
		}
		
		/// <summary>
		/// Gets whether list is supported.
		/// </summary>
		public bool CanList { get { return canList; } }
		/// <summary>
		/// Gets whether extract is supported.
		/// </summary>
		public bool CanExtract { get { return canExtract; } }
		/// <summary>
		/// Gets whether add is supported.
		/// </summary>
		public bool CanAdd { get { return canAdd; } }		
		/// <summary>
		/// Gets whether the archiver support OnOverwrite event.
		/// CakArchiver will handle it if not.
		/// </summary>
		public bool CanAlarmOverwrite { get { return canAlarmOverwrite; } }
		
		/// <summary>
		/// Open an archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public virtual bool Open() { archiveOpened = true; return false; }
		/// <summary>
		/// Close opened archive.
		/// </summary>
		/// <returns>Whether operation successful.</returns>
		public virtual bool Close() { archiveOpened = false; return true; }
		/// <summary>
		/// Get a list of supported extensions. 
		/// </summary>
		/// <returns>Support extension list (e.g. ".zip")</returns>
		public List<string> SupportedExtensions() 
		{
			List<string> retVal = new List<string>();
			
			foreach (ArchiveType at in SupportedArchiveTypes())
			{
				string[] exts = at.Extensions.Split(',');
				foreach (string ext in exts)
					if (retVal.IndexOf("." + ext) == -1)
					retVal.Add("." + ext);
			}
			
			return retVal; 
		}
		/// <summary>
		/// Get a list of supported archive types.
		/// </summary>
		/// <returns></returns>
		public virtual List<ArchiveType> SupportedArchiveTypes() { return new List<ArchiveType>(); }		
		/// <summary>
		/// Extract file from archive.
		/// </summary>
		/// <param name="options">Extract options.</param>
		/// <returns>Whether operation successful.</returns>
		public virtual bool Extract(ExtractOptions options) { return false; }
		/// <summary>
		/// Delete file from archive.
		/// </summary>
		/// <param name="options">Delete options.</param>
		/// <returns>Whether operation successful.</returns>
		public virtual bool Delete(DeleteOptions options) { return false; }
		/// <summary>
		/// Add file to archive.
		/// </summary>
		/// <param name="options">Delete options.</param>
		/// <returns>Whether operation successful.</returns>
		public virtual bool Add(AddOptions options) { return false; }
		/// <summary>
		/// Test integrity of the archive
		/// </summary>
		/// <returns>Whether the archive is valid.</returns>
		public virtual bool Test() { return false; }
		
		/// <summary>
		/// Create sfx of an archive
		/// </summary>
		/// <param name="options">SFX options</param>
		/// <returns>Whether operation successful.</returns>
		public virtual bool SFX(SFXOptions options) { return false; }
		
		/// <summary>
		/// List an archive.
		/// </summary>
		/// <param name="mask">File list mask.</param>
		/// <returns>Whether operation successful.</returns>
		public virtual bool List(string mask) { return false; }
		/// <summary>
		/// Gets if the archiver is working.
		/// </summary>
		/// <returns>Whether the archiver is working.</returns>
		public virtual bool Busy() { return false; }
				
		
		
		private Int32 supportCount;
		private void supportOnItemList(object sender, ItemListEventArgs e) {supportCount += 1; }
			
		public virtual bool Support(string anArchive)
		{			
			if (SupportedExtensions().IndexOf(Utils.ExtractFileExt(anArchive.ToLower())) != -1)
			{
				try
				{
					Debug.WriteLine("Testing support for " + this.name);
					Int32 count = 0;
					ArchiveName = anArchive;
					supportCount = 0;
					OnItemList += new ItemListEventHandler(supportOnItemList);
					Open();
					List("*");
					bool retVal = (supportCount > 0);
					Close();
					return retVal;
				}
				catch
				{
					OnItemList -= new ItemListEventHandler(supportOnItemList);
					return false;
				}
			}
			return false;
		}
		
		internal string GetMessage(ErrorMessage err, string filename)
		{
			string ProgressStr = err.ErrorDescription;
			return string.Format(ProgressStr, filename);
		}
		
		internal void DispProgress(ushort percent, string filename)
		{
			if (OnProgress != null)
				OnProgress(this, new ProgressEventArgs(this, percent, filename));
		}
		
		internal void DispMessage(ErrorMessage err)
		{
			
			if (OnMessage != null)
				OnMessage(this, new MessageEventArgs(this, GetMessage(err, "")));
		}
		internal void DispMessage(ErrorMessage err, string filename)
		{
			filename = Utils.ExtractFileName(filename);
			if (OnMessage != null)
				OnMessage(this, new MessageEventArgs(this, GetMessage(err, filename)));
		}
		internal void DispError(ErrorMessage err, string filename)
		{
			filename = Utils.ExtractFileName(filename);
			if (OnError != null)
				OnError(this, new ErrorEventArgs(this, GetMessage(err, filename)));
		}
		internal void DispError(string err, string filename)
		{
			filename = Utils.ExtractFileName(filename);
			if (OnError != null)
				OnError(this, new ErrorEventArgs(this, err + "(" + filename + ")"));
		}
	}
}
