using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Collections;
using System.Xml;
using QuickZip.ThemeBuilder;

namespace QuickZip.InstallBuilder
{
    /// <summary>
    /// Represent the first 100 byte of header file.
    /// </summary>
    internal class ProjectData
    {
        public String Signature = "MSCE";                           //Signature 
        public Int32 Unknown_Offset_4 = 0;                          //Unknown, usually 0s
        public Int32 HeaderLength;                                  //Size of header file (.000)
        public Int32 Unknown_Offset_12 = 0;                         //Unknown, usually 0s
        public Int32 Unknown_Offset_16 = 1;                         //Unknown, usually 1s
        public Processor TargetArchitecture = Processor.Unknown;    //Target processor type
        public Int32 MinMajorVersion = 0;                           //Minium major version
        public Int32 MinMinorVersion = 0;                           //Minium minor version
        public Int32 MaxMajorVersion = 0;                           //Maximum major version
        public Int32 MaxMinorVersion = 0;                           //Maximum minor version
        public Int32 MinBuild = 0;                                  //Minium build number
        public Int32 MaxBuild = 0;                                  //Maximum build number
        public StringSection Strings;                               //[Strings] Section
        public DirSection Dirs;                                     //[Dirs] Section
        public FileSection Files;                                   //[Files] Section
        public RegHivesSection RegHives;                            //[RegHives] Section
        public RegKeysSection RegKeys;                              //[RegKeys] Section
        public LinksSection Links;                                  //[Links] Section
        public StringInfo AppName = new StringInfo();               //Application Name
        public StringInfo Provider = new StringInfo();              //Author
        public StringInfo MultiString = new StringInfo();           //Unsupported system
        public Int16 Unknown_Offset_96 = 0;                         //Unknown, usually 0s            
        public Int16 Unknown_Offset_98 = 0;                         //Unknown usually 0s    
    
        //Inno Related
        public String Description = "";

        public ProjectData()
        {
            Strings = new StringSection(this);
            Dirs = new DirSection(this);
            Files = new FileSection(this);
            RegHives = new RegHivesSection(this);
            RegKeys = new RegKeysSection(this);
            Links = new LinksSection(this);
        }

        public String InstallDir
        {
            get
            {
                if (Strings.ID(1) != null)
                {
                    return Strings.ID(1).Value;
                }
                else
                {
                    return "";
                }
            }
            set
            {
                String input = DirEntry.EncodeDirectory(value,@"%CE1%\"+AppName.Value);
                if (input.EndsWith("\\"))
                    input = input.Substring(0,input.Length-1);

                input = DirEntry.EncodeDirectory(input);
                if (Strings.ID(1) != null)
                {
                    Strings.ID(1).Value = input;
                }
                else
                {
                    StringEntry se = new StringEntry(1, input);
                    Strings.Add(se);
                    Strings.EntriesCount += 1;
                }
            }
        }
    }

    /// <summary>
    /// Abstract Install project, hold ProjectData
    /// </summary>
    internal class InstallProject
    {
        public ProjectData Data;

        public String ProductName
        {
            get
            {
                return Data.AppName.Value;
            }
        }

        public String Description
        {
            get
            {
                return Data.Description;
            }
        }

        public InstallProject()
        {
            Data = new ProjectData();
        }

        public InstallProject(ProjectData value)
        {
            Data = value;
        }

   

        public String Byte2String(Byte[] input)
        {
            string s = "";
            for (int i = 0; i < input.Length; i++)
            {
                s = s + Convert.ToString(input[i]);
            }
            return s;            
        }
        public virtual void DebugClass()
        {
            Debug.WriteLine("Signature     = " + Data.Signature);
            Debug.WriteLine("Unknown(0)    = " + Data.Unknown_Offset_4.ToString());
            Debug.WriteLine("Header Length = " + Data.HeaderLength.ToString());
            Debug.WriteLine("Unknown(0)    = " + Data.Unknown_Offset_12.ToString());
            Debug.WriteLine("Unknown(1)    = " + Data.Unknown_Offset_16.ToString());
            Debug.WriteLine("Architecture  = " + Data.TargetArchitecture);
            Debug.WriteLine("MinVersion    = " + Data.MinMajorVersion.ToString() + "." + Data.MinMinorVersion.ToString());
            Debug.WriteLine("MaxVersion    = " + Data.MaxMajorVersion.ToString() + "." + Data.MaxMinorVersion.ToString());
            Debug.WriteLine("MinBuild      = " + Data.MinBuild.ToString());
            Debug.WriteLine("MaxBuild      = " + Data.MaxBuild.ToString());
            Debug.WriteLine("Unknown(0)    = " + Data.Unknown_Offset_96.ToString());
            Debug.WriteLine("Unknown(0)    = " + Data.Unknown_Offset_98.ToString());

            Data.AppName.DebugClass("AppName");
            Data.Provider.DebugClass("Provider");
            Data.MultiString.DebugClass("MultiString");

            Data.Strings.DebugClass("Strings");
            Data.Dirs.DebugClass("Dirs");
            Data.Files.DebugClass("Files");
            Data.RegHives.DebugClass("RegHives");
            Data.RegKeys.DebugClass("RegKeys");
            Data.Links.DebugClass("Links");
             
            Debug.Flush();
        }

        public Int32 RecalculateOffset()
        {
            Data.AppName.Offset = 100;
            Data.Provider.Offset = (Int16)(Data.AppName.Offset + Data.AppName.MinLength());
            Data.MultiString.Offset = (Int16)(Data.Provider.Offset + Data.Provider.MinLength());
            Data.Strings.Offset = (Int16)(Data.MultiString.Offset + Data.MultiString.MinLength());
            Data.Dirs.Offset = (Int16)(Data.Strings.Offset + Data.Strings.MinLength());
            Data.Files.Offset = (Int16)(Data.Dirs.Offset + Data.Dirs.MinLength());
            Data.RegHives.Offset = (Int16)(Data.Files.Offset + Data.Files.MinLength());
            Data.RegKeys.Offset = (Int16)(Data.RegHives.Offset + Data.RegHives.MinLength());
            Data.Links.Offset = (Int16)(Data.RegKeys.Offset + Data.RegKeys.MinLength());
            return (Int16)(Data.Links.Offset + Data.Links.MinLength());
        }

    }

    /// <summary>
    /// Encoded Inf Install project, read and write .000 Stream
    /// </summary>
    internal class Encoded_INF : InstallProject
    {
        public Encoded_INF()
        {
            Data = new ProjectData();
        }

        public Encoded_INF(ProjectData value) 
        {
            Data = value;
        }

        #region Read and Write Sections
        #region Strings Section
        public void ReadStrings(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.Strings.Offset;
            for (int i = 0; i < Data.Strings.EntriesCount; i++)
            {
                StringEntry se = new StringEntry();
                se.StringID = FileTools.ReadInt16(reader);
                se.ValueLength = FileTools.ReadInt16(reader);
                se.Value = FileTools.ReadString(reader, se.ValueLength);

                Data.Strings.Add(se);
            }
        }

        public void WriteStrings(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.Strings.Offset)
            {                
                throw new FormatException("Strings.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.Strings.Count; i++)
                {
                    StringEntry se = Data.Strings.String((Int16)i);
                    FileTools.WriteInt16(writer,se.StringID);
                    FileTools.WriteInt16(writer, se.ValueLength);
                    FileTools.WriteString(writer, se.Value);
                }
            }
        }
        #endregion
        #region Dirs Section
        public void ReadDirs(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.Dirs.Offset;
            for (int i = 0; i < Data.Dirs.EntriesCount; i++)
            {
                DirEntry de = new DirEntry();
                de.DirID = FileTools.ReadInt16(reader);
                de.ValueLength = FileTools.ReadInt16(reader);

                de.Value = new Int16[de.ValueLength / 2];
                for (int j = 0; j < (de.ValueLength / 2); j++)
                {
                    de.Value[j] = FileTools.ReadInt16(reader);
                }
                Data.Dirs.Add(de);
            }
        }
        public void WriteDirs(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.Dirs.Offset)
            {
                throw new FormatException("Dirs.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.Dirs.Count; i++)
                {
                    DirEntry de = Data.Dirs.Dir((Int16)i);
                    FileTools.WriteInt16(writer, de.DirID);
                    FileTools.WriteInt16(writer, de.ValueLength);
                    for (int j = 0; j < (de.ValueLength / 2); j++)
                    {
                        FileTools.WriteInt16(writer, de.Value[j]);
                    }

                }
            }
        }
        #endregion
        #region Files Section
        public void ReadFiles(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.Files.Offset;
            for (int i = 0; i < Data.Files.EntriesCount; i++)
            {
                FileEntry fe = new FileEntry();
                fe.FileID = FileTools.ReadInt16(reader);
                fe.DirID = FileTools.ReadInt16(reader);
                fe.FileNumber = FileTools.ReadInt16(reader);
                fe.Flags = FileTools.ReadUInt32(reader);
                fe.ValueLength = reader.ReadInt16();
                fe.Value = FileTools.ReadString(reader,fe.ValueLength);
                Data.Files.Add(fe);
            }
        }
        public void WriteFiles(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.Files.Offset)
            {
                throw new FormatException("Files.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.Files.Count; i++)
                {
                    FileEntry fe = Data.Files.File((Int16)i);
                    FileTools.WriteInt16(writer, fe.FileID);
                    FileTools.WriteInt16(writer, fe.DirID);
                    FileTools.WriteInt16(writer, fe.FileNumber);
                    FileTools.WriteUInt32(writer, fe.Flags);
                    FileTools.WriteInt16(writer, fe.ValueLength);
                    FileTools.WriteString(writer, fe.Value);
                }
            }
        }
        #endregion
        #region RegHives Section
        public void ReadRegHives(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.RegHives.Offset;
            for (int i = 0; i < Data.RegHives.EntriesCount; i++)
            {
                RegHiveEntry re = new RegHiveEntry();
                re.HiveID = FileTools.ReadInt16(reader);
                re.HiveRoot = (RegistryRoot)FileTools.ReadInt16(reader);
                re.HiveUnknown = FileTools.ReadInt16(reader);
                re.ValueLength = FileTools.ReadInt16(reader);
                re.Value = new Int16[re.ValueLength / 2];
                for (int j = 0; j < (re.ValueLength / 2); j++)
                {
                    re.Value[j] = FileTools.ReadInt16(reader);
                }
                Data.RegHives.Add(re);
            }
        }
        public void WriteRegHives(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.RegHives.Offset)
            {
                throw new FormatException("RegHives.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.RegHives.Count; i++)
                {
                    RegHiveEntry re = Data.RegHives.RegHive((Int16)i);
                    FileTools.WriteInt16(writer, re.HiveID);
                    FileTools.WriteInt16(writer, (Int16)re.HiveRoot);
                    FileTools.WriteInt16(writer, re.HiveUnknown);
                    FileTools.WriteInt16(writer, re.ValueLength);

                    for (int j = 0; j < (re.ValueLength / 2); j++)
                    {
                        FileTools.WriteInt16(writer, re.Value[j]);                         
                    }
                }
            }
        }
        #endregion
        #region RegKeys Section
        public void ReadRegKeys(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.RegKeys.Offset;
            for (int i = 0; i < Data.RegKeys.EntriesCount; i++)
            {
                RegKeyEntry re = new RegKeyEntry();
                re.KeyID = FileTools.ReadInt16(reader);
                re.HiveID = FileTools.ReadInt16(reader);
                re.KeyUnknown = FileTools.ReadInt16(reader);
                re.KeyDataType = FileTools.ReadInt32(reader);
                re.ValueLength = FileTools.ReadInt16(reader);

                re.KeyName = FileTools.ReadString(reader);

                switch (re.KeyDataType)
                {
                    case (0x00010001): //DWORD
                        re.Value_DWORD = FileTools.ReadInt32(reader);
                        break;
                    case 0x00000000: //SZ 
                        re.Value_String = FileTools.ReadString(reader, re.ValueLength - re.KeyName.Length - 1);
                        break;
                    case 0x00001000: //MULTI_SZ :
                        ArrayList list = new ArrayList();
                        String k = FileTools.ReadString(reader);
                        while (k != "")
                        {
                            list.Add(k);
                            k = FileTools.ReadString(reader);
                        }
                        re.Value_Multi_String = (String[])list.ToArray(typeof(String));
                        break;
                    case 0x00000001:  //Binary
                        re.Value_Binary = FileTools.ReadBytes(reader, re.ValueLength - re.KeyName.Length - 1);
                        break;
                }

                Data.RegKeys.Add(re);
            }
        }
        public void WriteRegKeys(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.RegKeys.Offset)
            {
                throw new FormatException("RegKeys.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.RegKeys.Count; i++)
                {
                    RegKeyEntry re = Data.RegKeys.RegKey((Int16)i);
                    FileTools.WriteInt16(writer, re.KeyID);
                    FileTools.WriteInt16(writer, re.HiveID);
                    FileTools.WriteInt16(writer, re.KeyUnknown);
                    FileTools.WriteInt32(writer, re.KeyDataType);
                    FileTools.WriteInt16(writer, re.ValueLength);
                    FileTools.WriteString(writer, re.KeyName);
                    switch (re.KeyDataType)
                    {
                        case (0x00010001): //DWORD
                            FileTools.WriteInt32(writer, re.Value_DWORD);                            
                            break;
                        case 0x00000000: //SZ 
                            FileTools.WriteString(writer, re.Value_String);                             
                            break;
                        case 0x00001000: //MULTI_SZ :
                            for (int j = 0; j < re.Value_Multi_String.Length; j++)
                            {
                                FileTools.WriteString(writer, re.Value_Multi_String[j]);
                            }
                            break;
                        case 0x00000001:  //Binary
                            FileTools.WriteBytes(writer, re.Value_Binary);                            
                            break;
                    }
                }
            }
        }
        #endregion
        #region Links Section
        public void ReadLinks(BinaryReader reader)
        {
            reader.BaseStream.Position = Data.Links.Offset;
            for (int i = 0; i < Data.Links.EntriesCount; i++)
            {
                LinkEntry le = new LinkEntry();
                le.LinkID = FileTools.ReadInt16(reader);
                le.LinkUnknown = FileTools.ReadInt16(reader);
                le.LinkBaseDir = (PPC30)FileTools.ReadInt16(reader);
                le.LinkTargetID = FileTools.ReadInt16(reader);
                le.LinkTargetType = (LinkType)FileTools.ReadInt16(reader);
                le.ValueLength = FileTools.ReadInt16(reader);
                le.Value = new Int16[le.ValueLength / 2];
                for (int j = 0; j < (le.ValueLength / 2); j++)
                {
                    le.Value[j] = FileTools.ReadInt16(reader);
                }
                Data.Links.Add(le);
            }
        }
        public void WriteLinks(BinaryWriter writer)
        {
            if (writer.BaseStream.Position != Data.Links.Offset)
            {
                throw new FormatException("Links.Offset dont match Writer's position!");
            }
            else
            {
                for (int i = 0; i < Data.Links.Count; i++)
                {
                    LinkEntry le = Data.Links.Link((Int16)i);
                    FileTools.WriteInt16(writer, le.LinkID);
                    FileTools.WriteInt16(writer, le.LinkUnknown);
                    FileTools.WriteInt16(writer, (Int16)le.LinkBaseDir);
                    FileTools.WriteInt16(writer, le.LinkTargetID);
                    FileTools.WriteInt16(writer, (Int16)le.LinkTargetType);
                    FileTools.WriteInt16(writer, le.ValueLength);
                    for (int j = 0; j < (le.ValueLength / 2); j++)
                    {
                        FileTools.WriteInt16(writer, le.Value[j]);                         
                    }                    
                }
            }
        }
        #endregion

        public void ReadStringInfo(StringInfo s, BinaryReader reader)
        {
            reader.BaseStream.Position = s.Offset;
            s.Value = FileTools.ReadString(reader, s.ValueLength-1);
        }
        #endregion 

        public void Read(BinaryReader reader)
        {
            Data.Signature = FileTools.ReadString(reader, 4);
            Data.Unknown_Offset_4 = FileTools.ReadInt32(reader);
            Data.HeaderLength = FileTools.ReadInt32(reader);
            Data.Unknown_Offset_12 = FileTools.ReadInt32(reader);
            Data.Unknown_Offset_16 = FileTools.ReadInt32(reader);
            Data.TargetArchitecture = (Processor)FileTools.ReadInt32(reader);
            Data.MinMajorVersion = FileTools.ReadInt32(reader);
            Data.MinMinorVersion = FileTools.ReadInt32(reader);
            Data.MaxMajorVersion = FileTools.ReadInt32(reader);
            Data.MaxMinorVersion = FileTools.ReadInt32(reader);
            Data.MinBuild = FileTools.ReadInt32(reader);
            Data.MaxBuild = FileTools.ReadInt32(reader);

            Data.Strings.EntriesCount = FileTools.ReadInt16(reader);
            Data.Dirs.EntriesCount = FileTools.ReadInt16(reader);
            Data.Files.EntriesCount = FileTools.ReadInt16(reader);
            Data.RegHives.EntriesCount = FileTools.ReadInt16(reader);
            Data.RegKeys.EntriesCount = FileTools.ReadInt16(reader);
            Data.Links.EntriesCount = FileTools.ReadInt16(reader);

            Data.Strings.Offset = FileTools.ReadInt32(reader);
            Data.Dirs.Offset = FileTools.ReadInt32(reader);
            Data.Files.Offset = FileTools.ReadInt32(reader);
            Data.RegHives.Offset = FileTools.ReadInt32(reader);
            Data.RegKeys.Offset = FileTools.ReadInt32(reader);
            Data.Links.Offset = FileTools.ReadInt32(reader);

            Data.AppName.Offset = FileTools.ReadInt16(reader);
            Data.AppName.ValueLength = FileTools.ReadInt16(reader);
            Data.Provider.Offset = FileTools.ReadInt16(reader);
            Data.Provider.ValueLength = FileTools.ReadInt16(reader);
            Data.MultiString.Offset = FileTools.ReadInt16(reader);
            Data.MultiString.ValueLength = FileTools.ReadInt16(reader);

            Data.Unknown_Offset_96 = FileTools.ReadInt16(reader);
            Data.Unknown_Offset_98 = FileTools.ReadInt16(reader);

            ReadStringInfo(Data.AppName, reader);
            ReadStringInfo(Data.Provider, reader);
            ReadStringInfo(Data.MultiString, reader);

            ReadStrings(reader);
            ReadDirs(reader);
            ReadFiles(reader);
            ReadRegHives(reader);
            ReadRegKeys(reader);
            ReadLinks(reader);

        }

        public void Write(BinaryWriter writer)
        {
            FileTools.WriteString(writer, Data.Signature, 4);
            FileTools.WriteInt32(writer, Data.Unknown_Offset_4);
            FileTools.WriteInt32(writer, RecalculateOffset());
            FileTools.WriteInt32(writer, Data.Unknown_Offset_12);
            FileTools.WriteInt32(writer, Data.Unknown_Offset_16);
            FileTools.WriteInt32(writer, (Int32)Data.TargetArchitecture);
            FileTools.WriteInt32(writer, Data.MinMajorVersion);
            FileTools.WriteInt32(writer, Data.MinMinorVersion);
            FileTools.WriteInt32(writer, Data.MaxMajorVersion);
            FileTools.WriteInt32(writer, Data.MaxMinorVersion);
            FileTools.WriteInt32(writer, Data.MinBuild);
            FileTools.WriteInt32(writer, Data.MaxBuild);

            FileTools.WriteInt16(writer, Data.Strings.EntriesCount);
            FileTools.WriteInt16(writer, Data.Dirs.EntriesCount);
            FileTools.WriteInt16(writer, Data.Files.EntriesCount);
            FileTools.WriteInt16(writer, Data.RegHives.EntriesCount);
            FileTools.WriteInt16(writer, Data.RegKeys.EntriesCount);
            FileTools.WriteInt16(writer, Data.Links.EntriesCount);

            FileTools.WriteInt32(writer, Data.Strings.Offset);
            FileTools.WriteInt32(writer, Data.Dirs.Offset);
            FileTools.WriteInt32(writer, Data.Files.Offset);
            FileTools.WriteInt32(writer, Data.RegHives.Offset);
            FileTools.WriteInt32(writer, Data.RegKeys.Offset);
            FileTools.WriteInt32(writer, Data.Links.Offset);

            FileTools.WriteInt16(writer, Data.AppName.Offset);
            FileTools.WriteInt16(writer, Data.AppName.ValueLength);
            FileTools.WriteInt16(writer, Data.Provider.Offset);
            FileTools.WriteInt16(writer, Data.Provider.ValueLength);
            FileTools.WriteInt16(writer, Data.MultiString.Offset);
            FileTools.WriteInt16(writer, Data.MultiString.ValueLength);

            FileTools.WriteInt16(writer, Data.Unknown_Offset_96);
            FileTools.WriteInt16(writer, Data.Unknown_Offset_98);

            if (writer.BaseStream.Position != 100)
            {
                throw new FormatException("Something goes wrong, Position != 100!");
            }

            FileTools.WriteString(writer, Data.AppName.Value);
            FileTools.WriteString(writer, Data.Provider.Value);
            FileTools.WriteString(writer, Data.MultiString.Value);

            WriteStrings(writer);
            WriteDirs(writer);
            WriteFiles(writer);
            WriteRegHives(writer);
            WriteRegKeys(writer);
            WriteLinks(writer);
        }

        

    }

    /// <summary>
    /// Describe what the Install project does
    /// </summary>
    internal class Describe_INF : InstallProject
    {

        public Describe_INF()
        {
            Data = new ProjectData();
        }

        public Describe_INF(ProjectData value) 
        {
            Data = value;
        }

        public override void DebugClass()
        {
            Debug.WriteLine(""); Debug.WriteLine("-----  Debugging   -----");

            Debug.WriteLine(""); Debug.WriteLine("-----  Dir Section -----");
            Debug.WriteLine(
                String.Format("InstallDir = {0}",
                Data.InstallDir));
            for (int i = 0; i < Data.Dirs.Count; i++)
            {
                DirEntry de = Data.Dirs.Dir((Int16)i);
                String Directory = Data.Strings.ParseString(de.Value);

                Debug.WriteLine(
                String.Format("{0} = {1}", de.DirID, DirEntry.DecodeDirectory(Directory,Data.InstallDir))
                );
            }

            Debug.WriteLine(""); Debug.WriteLine("----- File Section -----");
            for (int i = 0; i < Data.Files.Count; i++)
            {
                FileEntry fe = Data.Files.File((Int16)i);                
                DirEntry de = Data.Dirs.ID(fe.DirID);
                
                String Directory = Data.Strings.ParseString(de.Value);
                String Filename = fe.Value;
                

                Debug.Write(
                String.Format("{0} --> {1} ({2}) ", Filename, DirEntry.DecodeDirectory(Directory, Data.InstallDir), fe.Type)
                );
                if (fe.IsSelfRegister)
                    Debug.Write('A');
                if (fe.IsShared)
                    Debug.Write('S');
                Debug.WriteLine("");
            }

            Debug.WriteLine(""); Debug.WriteLine("--- Registry Section ---");
            for (int i = 0; i < Data.RegKeys.Count; i++)
            {
                RegKeyEntry re = Data.RegKeys.RegKey((Int16)i);
                RegHiveEntry rhe = Data.RegHives.ID(re.HiveID);

                String Root = Convert.ToString((RegistryRoot)(rhe.HiveRoot));
                String Path = Data.Strings.ParseString(rhe.Value);
                String Key = re.KeyName;
                String Value = re.DebugValue;
                String Type = Convert.ToString((RegistryDataType)(re.KeyDataType));              

                Debug.WriteLine(
                String.Format("{0},{1},{2} = '{3}'({4})", 
                    Root, Path, Key, Value, Type)
                );
            }

            Debug.WriteLine(""); Debug.WriteLine("----- Link Section -----");
            for (int i = 0; i < Data.Links.Count; i++)
            {
                LinkEntry le = Data.Links.Link((Int16)i);
                FileEntry fe = Data.Files.ID(le.LinkTargetID);
                DirEntry de = Data.Dirs.ID(fe.DirID);

                
                String BaseDir = Convert.ToString((PPC30)le.LinkBaseDir);
                String Specification = Data.Strings.ParseString(le.Value);                
                String Target = Data.Strings.ParseString(de.Value) + @"\" + fe.Value;
                String LinkTyp = Convert.ToString((LinkType)le.LinkTargetType);

                Debug.WriteLine(
                String.Format("{0},{1} = '{2}'({3})",
                    BaseDir, Specification, DirEntry.DecodeDirectory(Target, Data.InstallDir), LinkTyp)
                );
            }

            Debug.WriteLine(""); Debug.WriteLine("-----   End Debug  -----");
        }
    }

    /// <summary>
    /// Inno Install project, read and write Inno Stream
    /// </summary>
    internal class InnoProject : InstallProject
    {
        ArrayList Setup = new ArrayList();
        ArrayList Files = new ArrayList();
        ArrayList Icons = new ArrayList();
        ArrayList Registry = new ArrayList();
        ArrayList Unknown = new ArrayList();

        public SetupFileType SetupFile = SetupFileType.EncodedXml;

        //public Hashtable FileRenameList = new Hashtable();
        //public ArrayList FileIDList = new ArrayList();
        //public ArrayList FullNameList = new ArrayList();

        public InnoProject()
        {
            Data = new ProjectData();
        }

        public InnoProject(ProjectData value) 
        {
            Data = value;
        }

        #region read and write section
        public bool IsSection(String value)
        {
            if (value.IndexOf("[") == 0)
                if (value.IndexOf("]") != -1)
                    return true;
            return false;
        }

        public String RemoveComments(String input)
        {
            if (input.Trim().StartsWith(";"))
                return "";

            Int32 CommentPos = input.IndexOf("//");
            if (CommentPos != -1)
                return (input.Substring(0, CommentPos-1)).Trim();

            return input.Trim();
        }
        public void ReadSections(StreamReader reader)
        {
            ArrayList Current = Unknown;
            String input;
            Setup.Clear();
            Files.Clear();
            Icons.Clear();
            Registry.Clear();
            Unknown.Clear();

            while (!reader.EndOfStream)
            {
                input = RemoveComments(reader.ReadLine());
                String sectionstring = input.ToLower().Trim();
                if (sectionstring == "[setup]") { Current = Setup; }
                else
                    if (sectionstring == "[files]") { Current = Files; }
                    else
                        if (sectionstring == "[icons]") { Current = Icons; }
                        else
                            if (sectionstring == "[registry]") { Current = Registry; }
                            else
                                if (IsSection(sectionstring)) { Current = Unknown; }
                                else Current.Add(input);                                
                            
            }

        }

        public void ReadSetupSection()
        {
            for (int i = 0; i < Setup.Count; i++)
            {
                String currentitem = (String)(Setup[i]);
                if (currentitem.IndexOf("=") != -1)
                {
                    String Key = currentitem.Substring(0, currentitem.IndexOf("="));
                    String Value = currentitem.Substring(Key.Length + 1, currentitem.Length - Key.Length - 1);

                    if (Key.ToLower() == "appname") { Data.AppName.Value = Value; }
                    else if (Key.ToLower() == "author") { Data.Provider.Value = Value; }
                    else if (Key.ToLower() == "defaultdirname") {Data.InstallDir = Value; }
                    else if (Key.ToLower() == "description") { Data.Description = Value; }
                    else if (Key.ToLower() == "targetplatform") { Data.TargetArchitecture = Tools.Str2ProcessorType(Value); }
                    else if (Key.ToLower() == "setupfile") { SetupFile = Tools.Str2SetupFileType(Value); }
                }
            }
        }

        public void WriteSetupSection(StreamWriter writer)
        {
            
            writer.Write("[Setup]\r\n");
            writer.WriteLine(String.Format("AppName={0}", Data.AppName.Value));
            writer.WriteLine(String.Format("Author={0}", Data.Provider.Value));
            writer.WriteLine(String.Format("TargetPlatform={0}", Convert.ToString(Data.TargetArchitecture)));
            
            StringEntry se = Data.Strings.ID(1);
            String DefaultDirName;
            if (se != null)
            {
                DefaultDirName = DirEntry.DecodeDirectory2(se.Value,Data.InstallDir);                
            }
            else
            {
                DefaultDirName = @"{pf}\" + Data.AppName;
            }
            writer.WriteLine(String.Format("DefaultDirName={0}", DefaultDirName));
            writer.WriteLine("");            
        }

        public static Hashtable ReadEntry(String line)
        {
            Hashtable temp = new Hashtable();

            String[] keypairs = line.Split(new Char [] {';'});
            foreach (String s in keypairs)
            {
                String[] keypair = s.Split(new Char[] { ':' }, 2);
                if (keypair.Length >= 2)
                {
                    String Key = keypair[0].Trim().ToLower();
                    String Value = keypair[1].Trim();
                    if (Value[0] == '\"')
                    {
                        Value = Value.Substring(1, Value.Length - 2);
                    }
                    temp.Add(Key, Value);
                }
            }

            return temp;
        }

        public void ReadFilesSection()
        {
            for (int i = 0; i < Files.Count; i++)
            {
                String Line = (String)Files[i];
                String Source = "";
                String Destdir = "";
                String Destfile = "";
                String Flags = "";
                Hashtable Entry = ReadEntry(Line);
                foreach (DictionaryEntry de in Entry)
                {                    
                    if ((String)de.Key == "source") { Source = (String)de.Value; }
                    else
                        if ((String)de.Key == "destdir") { Destdir = (String)de.Value; }                                        
                        else
                            if ((String)de.Key == "destfile") { Destfile = (String)de.Value; }
                            else
                                if ((String)de.Key == "flags") { Flags = ((String)de.Value).ToLower(); }
                }
                if (Source != "")
                    if (Destdir != "")     
                    {                                                
                        DirectoryInfo di = new DirectoryInfo(Tools.ExtractFilePath(Source));
                        FileInfo[] files = di.GetFiles(Tools.ExtractFileName(Source));                        
                        UInt32 flags = 0;

                        if (Flags.IndexOf("noskip") > -1) { flags = (UInt32)FileEntry.FileEntryType.NoSkip; }
                        else if (Flags.IndexOf("nooverwrite") > -1) { flags = (UInt32)FileEntry.FileEntryType.NoOverwrite; }
                        else if (Flags.IndexOf("overwriteifexists") > -1) { flags = (UInt32)FileEntry.FileEntryType.OverwriteIfExists; }
                        else if (Flags.IndexOf("nooverwriteifnewer") > -1) { flags = (UInt32)FileEntry.FileEntryType.NoOverwriteIfNewer; }
                        else if (Flags.IndexOf("nodatecheck") > -1) { flags = (UInt32)FileEntry.FileEntryType.NoDateCheck; }

                        if (Flags.IndexOf("selfregister") > -1) { flags += (UInt32)FileEntry.FileEntryType.SelfRegister; }
                        if (Flags.IndexOf("shared") > -1) { flags += (UInt32)FileEntry.FileEntryType.Shared; }

                        if (files.Length == 0)
                        {
                            throw new FileNotFoundException(Source + " Not Found!", Source);
                        }
                        if ((Destfile != "") && (files.Length > 1))
                        {
                            throw new Exception("DestFile not supported for Source with Wildcards!" );
                        }

                        foreach (FileInfo fi in files)
                        {
                            //Debug.WriteLine(fi.FullName);                            
                            String NewSource = (Tools.ExtractFileName(fi.FullName));
                            String NewDest = "";

                            if (Destfile != "") { NewDest = Destfile; } else { NewDest = NewSource; }

                            Data.Files.AddFile(Destdir, ref NewDest, Source, flags);

                            if (Destfile != "") { NewDest = Destfile; } else { NewDest = NewSource; }
                            NewSource = (Destdir + "\\" + NewDest);
                      
                            NewSource = DirEntry.EncodeDirectory(NewSource, Data.InstallDir);
                        }

                        
                        
                    }                
            }
        }

        public void WriteFilesSection(StreamWriter writer)
        {
            writer.WriteLine("[Files]");
            for (int i = 0; i < Data.Files.Count; i++)
            {
                FileEntry fe = Data.Files.File((Int16)i);
                DirEntry de = Data.Dirs.ID(fe.DirID);
                String Directory = DirEntry.DecodeDirectory(
                   Data.Strings.ParseString(de.Value), Data.InstallDir);
                String FileName = fe.SourcePath;

                String Flags = "";
                if (fe.Type != FileEntry.FileEntryType.Normal)
                    Flags = Convert.ToString(fe.Type) + " ";
                if (fe.IsShared)
                    Flags += "Shared ";
                if (fe.IsSelfRegister)
                    Flags += "SelfRegister";
                Flags = Flags.Trim();

                if ((Flags != "") && (Flags != "0"))
                    writer.WriteLine(String.Format("Source: \"{0}\"; DestDir: \"{1}\"; DestFile:\"{2}\" Flags: \"{3}\"", FileName, Directory, fe.Value, Flags));
                else
                    writer.WriteLine(String.Format("Source: \"{0}\"; DestDir: \"{1}\"; DestFile:\"{2}\"", FileName, Directory, fe.Value));
            }
            writer.WriteLine(""); 
        }

        public void ReadIconsSection()
        {
            for (int i = 0; i < Icons.Count; i++)
            {
                String Line = (String)Icons[i];
                String Name = "";
                String Filename = "";
                Hashtable Entry = ReadEntry(Line);
                foreach (DictionaryEntry de in Entry)
                {

                    if ((String)de.Key == "name") { 
                        Name = (String)de.Value;
                        if (!Name.ToLower().EndsWith(".lnk"))
                            Name += ".lnk";
                        }
                    else
                        if ((String)de.Key == "filename") { Filename = (String)de.Value; }
                }
                if (Name != "")
                    if (Filename != "")
                    {
                        String path = DirEntry.EncodeDirectory(Filename, Data.InstallDir);
                        path = DirEntry.EncodeDirectory(path, Data.InstallDir);
                        Int16 ID = (Int16)((Data.Files.File(path)).FileID);
                        if (ID != 0)
                        {
                            PPC30 basedir;
                            String LinkTargetName =
                                DirEntry.DecodeDirectory(Name, "!@#$", out basedir);

                            Data.Links.AddLink(basedir, LinkType.file, ID, LinkTargetName);
                        }
                        else
                        {
                            throw new FormatException(Filename + " Not found in file section.");
                        }
                    }
            }
        }

        public void WriteIconsSection(StreamWriter writer)
        {
            writer.WriteLine("[Icons]");
            String path;
            for (int i = 0; i < Data.Links.Count; i++)
            {
                LinkEntry le = Data.Links.Link((Int16)i);
                if (le.LinkTargetType == LinkType.directory)
                {
                    DirEntry de = Data.Dirs.ID(le.LinkTargetID);
                    path = Data.Strings.ParseString(de.Value);
                }
                else
                {
                    FileEntry fe = Data.Files.ID(le.LinkTargetID);
                    DirEntry de = Data.Dirs.ID(fe.DirID);
                    path = Data.Strings.ParseString(de.Value) + @"\" + fe.Value;
                }                
                
                String linkpath = "{" + Convert.ToString((PPC30)le.LinkBaseDir) + @"}\" +  Data.Strings.ParseString(le.Value);
                path = DirEntry.DecodeDirectory(path, Data.InstallDir);
                linkpath = DirEntry.DecodeDirectory(linkpath, Data.InstallDir);

                writer.WriteLine(String.Format("Name: \"{0}\"; Filename: \"{1}\"", linkpath, path));
            }
            writer.WriteLine(""); 
        }

        public void ReadRegistrySection()
        {
            for (int i = 0; i < Registry.Count; i++)
            {
                String Line = (String)Registry[i];
                RegistryRoot Root = RegistryRoot.HKCU;
                String Subkey = "";
                Int32 ValueType = 0x00000000;
                String ValueName = "";
                String ValueData = "";

                Hashtable Entry = ReadEntry(Line);
                foreach (DictionaryEntry de in Entry)
                {
                    if ((String)de.Key == "root") { Root = Tools.Str2RegistryRoot(((String)de.Value)); }
                    else
                        if ((String)de.Key == "subkey") { Subkey = (String)de.Value; }
                        else
                            if ((String)de.Key == "valuetype") { ValueType = Tools.Str2KeyDataType((String)de.Value); }
                            else
                                if ((String)de.Key == "valuename") { ValueName = (String)de.Value; }
                                else
                                    if ((String)de.Key == "valuedata") { ValueData = (String)de.Value; }
                }

                if (Subkey != "")
                {
                    ValueData = ValueData.Replace("{app}","%InstallDir%");
                    Int32 value32 = 0;
                    String[] valueMs = new String[1];
                    Byte[] valueBin = new Byte[1];
                    switch (ValueType)
                    {
                        case 0x00010001: //DWORD
                            value32 = Convert.ToInt32(ValueData);
                            break;
                        case 0x00000000: //String
                            // do nothing
                            break;
                        case 0x00001000: //MULTI_SZ :                            
                            valueMs = FileTools.String2MultiString(ValueData);
                            break;
                        case 0x00000001:  //Binary
                            valueBin = FileTools.String2ByteArray(ValueData);
                            break;
                    }

                    Data.RegKeys.AddRegKey(Root, Subkey, ValueName, ValueType, value32, ValueData, valueMs, valueBin);
                }
            }
        }

        public void WriteRegistrySection(StreamWriter writer)
        {
            writer.WriteLine("[Registry]");

            for (int i = 0; i < Data.RegKeys.Count; i++)
            {
                RegKeyEntry re = Data.RegKeys.RegKey((Int16)i);
                RegHiveEntry rhe = Data.RegHives.ID(re.HiveID);

                String root = Convert.ToString((RegistryRoot)(rhe.HiveRoot));
                String valuetype = (Convert.ToString((KeyDataType)(re.KeyDataType))).Substring(1);
                String valuename = re.KeyName;
                String subkey = Data.Strings.ParseString(rhe.Value);

                String value;
                switch (re.KeyDataType)
                {
                    case 0x00010001 : //DWORD
                        value = Convert.ToString(re.Value_DWORD);
                        break;
                    case 0x00000000 : //String
                        value = re.Value_String;
                        break;
                    case 0x00001000: //MULTI_SZ 
                        value = FileTools.MultiString2String(re.Value_Multi_String);                        
                        break;
                    case 0x00000001:  //Binary
                        value = "";
                        foreach (byte b in re.Value_Binary)
                        {
                            value += Convert.ToString(b, 16).ToUpper();
                            if (value.Length % 2 == 1)
                                value += "0";
                        }                        
                        break;
                    default : 
                        value = "";
                        break;
                }

                value = DirEntry.DecodeDirectory(value, Data.InstallDir);                

                writer.WriteLine(String.Format(
                    "Root: {0}; Subkey: \"{1}\"; ValueType: {2}; ValueName: {3}; ValueData: \"{4}\"",
                    root,subkey,valuetype,valuename,value));
            }
            writer.WriteLine(""); 

        }
        #endregion
        public void Read(StreamReader reader)
        {
            ReadSections(reader);
            ReadSetupSection();
            ReadFilesSection();
            ReadIconsSection();
            ReadRegistrySection();
            Data.HeaderLength = RecalculateOffset();
        }

        public void Write(StreamWriter writer)
        {
            WriteSetupSection(writer);
            WriteFilesSection(writer);
            WriteIconsSection(writer);
            WriteRegistrySection(writer);
        }

        public void FileInfo(Int16 index, out String SourceFileName, out String RenameFileName, out String DestDir, out String DestFileName)
        {
            SourceFileName = Data.Files.File(index).SourcePath;
            RenameFileName = Data.Files.File(index).ShortPath;

            FileEntry fe = Data.Files.File(index);
            DirEntry de = Data.Dirs.ID(fe.DirID);

            DestDir = Data.Strings.ParseString(de.Value);
            DestFileName = fe.Value;           
        }
    }

    /// <summary>
    /// Generate and read (hopefully when I have time to code it) for Xml Install(_setup.xml)
    /// </summary>
    internal class XmlProject : InstallProject
    {
        public XmlProject()
        {
            Data = new ProjectData();
        }

        public XmlProject(ProjectData value) 
        {
            Data = value;
        }
                
        public void WriteParam(XmlWriter writer, String ParamName)
        {
            writer.WriteStartElement("parm");
            writer.WriteAttributeString("name", ParamName);            
            writer.WriteEndElement();
        }

        public void WriteParam(XmlWriter writer, String ParamName, String ParamValue)
        {
            writer.WriteStartElement("parm");
            writer.WriteAttributeString("name", ParamName);
            writer.WriteAttributeString("value", ParamValue);
            writer.WriteEndElement();
        }

        public void WriteParam(XmlWriter writer, String ParamName, String ParamValue,
               String SecondParamKey, String SecondParamValue)
        {

                writer.WriteStartElement("parm");
                writer.WriteAttributeString("name", ParamName);
                writer.WriteAttributeString("value", ParamValue);
                writer.WriteAttributeString(SecondParamKey, SecondParamValue);
                writer.WriteEndElement();

        }
        public void WriteParam(XmlWriter writer, String ParamName, String ParamValue,
               String SecondParamKey, String SecondParamValue, String ThirdParamKey, String ThirdParamValue)
        {

            writer.WriteStartElement("parm");
            writer.WriteAttributeString("name", ParamName);
            writer.WriteAttributeString("value", ParamValue);
            writer.WriteAttributeString(SecondParamKey, SecondParamValue);
            writer.WriteAttributeString(ThirdParamKey, ThirdParamValue);
            writer.WriteEndElement();

        }

        public void WriteElement(XmlWriter writer, String ElementName, String[] ParamNames, String[] ParamValues,
            Boolean EndElement)
        {
            writer.WriteStartElement(ElementName);
            for (int i = 0; i < ParamNames.Length; i++)
            {
                writer.WriteAttributeString(ParamNames[i], ParamValues[i]);
            }
            if (EndElement) writer.WriteEndElement();            
        }

        public void WriteSetupSection(XmlWriter writer)
        {                                         
            writer.WriteStartElement("characteristic");
            writer.WriteAttributeString("type", "Install");

            WriteParam(writer, "InstallPhase", "Install");
            WriteParam(writer, "OSVersionMin", Data.MinMajorVersion.ToString() + "." + Data.MinMinorVersion.ToString());
            WriteParam(writer, "OSVersionMax", Data.MaxMajorVersion.ToString() + "." + Data.MaxMinorVersion.ToString());

            WriteParam(writer, "BuildNumberMin", Data.MinBuild.ToString());
            WriteParam(writer, "BuildNumberMax", Data.MaxBuild.ToString());
            WriteParam(writer, "AppName", Data.Provider.Value + " " + Data.AppName.Value);
            WriteParam(writer, "InstallDir", Data.InstallDir, "translation", "install");
            WriteParam(writer, "NumDirs", Data.Dirs.Count.ToString());
            WriteParam(writer, "NumFiles", Data.Files.Count.ToString());
            WriteParam(writer, "NumRegKeys", Data.RegHives.Count.ToString());
            WriteParam(writer, "NumRegVals", Data.RegKeys.Count.ToString());
            WriteParam(writer, "NumShortcuts", Data.Links.Count.ToString());

            writer.WriteEndElement();

        }
        
        void SplitOSVersion(string version, out int major, out int minor)
        {
        	string[] temp = version.Split(new char[] {'.'});
        	if (temp.Length != 2)
        		throw new FormatException("OSVersionString Format Error!");
        	major = Convert.ToInt32(temp[0]);
        	minor = Convert.ToInt32(temp[1]);
        }
        
        public void ReadSetupSection(XmlReader reader)
        {        	
        	XmlReader subTree = reader.ReadSubtree();
        	
        	while (subTree.Read())
            {                
        		if ((reader.NodeType == XmlNodeType.Element) && (reader.Name == "parm"))
                {
                	string name = reader.GetAttribute("name");
                	string value = reader.GetAttribute("value");
                	Data.InstallDir = "";
//                	Debug.WriteLine(name + "=" + value);
                	switch (name)
                	{
                			case "OSVersionMin" : 
                				SplitOSVersion(value, out Data.MinMajorVersion, out Data.MinMinorVersion);
                				break;
                			case "OSVersionMax" : 
                				SplitOSVersion(value, out Data.MaxMajorVersion, out Data.MaxMinorVersion);
                				break;
                			case "BuildNumberMin":
                				Data.MinBuild = Convert.ToInt32(value);
                				break;
                			case "BuildNumberMax":
                				Data.MaxBuild = Convert.ToInt32(value);
                				break;
                			case "AppName":
                				Data.AppName.Value = value;
                				break;
                			case "InstallDir":
                				Data.InstallDir = value;
                				break;                				                				
                	}
                }
            }        	
        }

        public void WriteFilesIconsSection(XmlWriter writer)
        {
            writer.WriteStartElement("characteristic");
            writer.WriteAttributeString("type", "FileOperation");
            for (int i = 0; i < Data.Dirs.Count; i++)
            {
                DirEntry de = Data.Dirs.Dir((Int16)i);

                string path;
                //Fix:0.6
//                if (Data.Strings.ParseString(de.Value) == Data.InstallDir)
//                path = "%InstallDir%" ;
//                else
                path = Data.Strings.ParseString(de.Value).Replace(Data.InstallDir, "%InstallDir%");
                
                WriteElement(writer, "characteristic", new String[2] { "type", "translation" },
                    new String[2] {path, "install"}, false);
                for (int j = 0; j < Data.Files.Count; j++)
                {
                    FileEntry fe = Data.Files.File((Int16)j);

                    WriteElement(writer, "characteristic", new String[1] { "type" },
                    new String[1] { "MakeDir" }, true);

                    if (fe.DirID == de.DirID)
                    {                       
                        WriteElement(writer, "characteristic", new String[2] { "type", "translation" },
                            new String[2] { fe.Value, "install" }, false);
                        
                        WriteElement(writer, "characteristic", new String[1] { "type" },
                            new String[1] { "Extract" }, false);

                        WriteParam(writer, "Source", Tools.GetDosFilename(fe.Value, fe.FileNumber));

                        if (fe.Type == FileEntry.FileEntryType.Normal)
//                            WriteParam(writer, Convert.ToString(fe.Type));
//                        else
                            WriteParam(writer, "WarnIfSkip");
                        
                        if (fe.IsShared)
                            WriteParam(writer, "Shared");
                        if (fe.IsSelfRegister)
                            WriteParam(writer, "SelfRegister");
                            
                        writer.WriteEndElement();
                    }
                    writer.WriteEndElement();
                }
                
                writer.WriteEndElement();
            }

            if (Data.Links.Count > 0)
            {                
                WriteElement(writer, "characteristic", new String[2] { "type", "translation" },
                        new String[2] { "%CE11%", "install" }, false);
                WriteElement(writer, "characteristic", new String[1] { "type" }, new String[1] { "MakeDir" }, true);
                for (int i = 0; i < Data.Links.Count; i++)
                {
                    LinkEntry le = Data.Links.Link((Int16)i);
                    FileEntry fe = Data.Files.ID((Int16)le.LinkTargetID);
                    DirEntry de = Data.Dirs.ID((Int16)fe.DirID);

                    WriteElement(writer, "characteristic", new String[2] { "type", "translation" },
                          new String[2] { Data.Strings.ParseString(le.Value),
                          "install" }, false);

                    WriteElement(writer, "characteristic", new String[1] { "type" }, new String[1] { "Shortcut" }, false);

                    
                    String Target = String.Format(@"{1}\{2}",
                            (Int32)le.LinkBaseDir,             //Fix:0.6: Shortcut not working if install in CF 
                            Data.Strings.ParseString(de.Value).Replace(Data.InstallDir, "%InstallDir%"), 
                            fe.Value);
                    WriteParam(writer, "Source",Target,"translation","install");

                    writer.WriteEndElement();

                    writer.WriteEndElement();
                }
                writer.WriteEndElement();
            }


            writer.WriteEndElement();
            
        }
        
        
        void ReadParam(XmlReader reader, out string type, out string[] names, out string[] values)
        {
        	List<string> nameList = new List<string>();
        	List<string> valueList = new List<string>();
        	
        	XmlReader subTree = reader.ReadSubtree();
        	type = "";
        	while (subTree.Read())
            {        
        		if (reader.GetAttribute("type") != null)
        			type = reader.GetAttribute("type");
        		if ((reader.NodeType == XmlNodeType.Element) && 
        		    (reader.Name == "parm"))
        		{
        			nameList.Add(reader.GetAttribute("name"));
        			valueList.Add(reader.GetAttribute("value"));
        		}
        	}
        	names = nameList.ToArray();
        	values = valueList.ToArray();
        }
        
        void ReadFileIconEntry(XmlReader reader, string dir)
        {        	
        	XmlReader subTree = reader.ReadSubtree();
        	subTree.Read();		//Read Directory name
        	
        	while (subTree.Read())
            {
        		if ((reader.NodeType == XmlNodeType.Element) && 
        		    (reader.GetAttribute("translation") == "install"))
        			{        				        				
        				string[] names;
        				string[] values;
        				string destFile = reader.GetAttribute("type");
        				string srcFile = "";       
        				string type;
        				uint flags = 0;        				
        				ReadParam(reader, out type, out names, out values);        				        				
//        				Debug.WriteLine(type);
        				for (int i = 0; i < names.Length; i++)
        				{
							string value = values[i];
        					switch (names[i])
        					{
        						case "Source": srcFile = value; break;
        						case "WarnIfSkip" :
        							flags += (uint)FileEntry.FileEntryType.NoSkip;
        							break;
        						case "Shared" :
	        						flags += (uint)FileEntry.FileEntryType.Shared;
    	    						break;
        						case "SelfRegister" :
    	    						flags += (uint)FileEntry.FileEntryType.SelfRegister;
        							break;
        					}
        				}
        				
//        				Debug.WriteLine(dir + " - " + srcFile + " - " + destFile);
        				
        				switch (type)
        				{
        					case "Extract":
        						if (srcFile != "")
        							Data.Files.AddFile(dir, ref destFile, srcFile, flags);
        						break;
        					case "Shortcut":
        						PPC30 basedir;
        						Int16 ID = (Int16)((Data.Files.File(srcFile)).FileID);
                            	String LinkTargetName =
                                	DirEntry.DecodeDirectory(destFile, "!@#$", out basedir);
                            	Data.Links.AddLink(basedir, LinkType.file, ID, LinkTargetName);
                            	break;
        				}
        				

        			}
        			
        	}
        }
        
        void ReadFileIconSection(XmlReader reader)
        {
        	string type = reader.GetAttribute("type");
        	string Dir = type;        	
        	XmlReader subTree = reader.ReadSubtree();        	
        	
        	while (subTree.Read())
            {
        		ReadFileIconEntry(subTree, Dir);
        	}
        }
        
        public void ReadFilesIconsSection(XmlReader reader)
        {
        	XmlReader subTree = reader.ReadSubtree();        	        	
        	
        	while (subTree.Read())
            {
        		if ((reader.NodeType == XmlNodeType.Element) && 
        		    (reader.GetAttribute("translation") == "install"))
                {
                	ReadFileIconSection(subTree);
        		}
        	}
        		
        }

        public void WriteRegistrySection(XmlWriter writer)
        {
            writer.WriteStartElement("characteristic");
            writer.WriteAttributeString("type", "Registry");

            for (int i = 0; i < Data.RegHives.Count; i++)
            {
                RegHiveEntry rhe = Data.RegHives.RegHive((Int16)i);
                String root = Convert.ToString(rhe.HiveRoot) + @"\" + Data.Strings.ParseString(rhe.Value);
                WriteElement(writer, "characteristic", new String[1] { "type" }, 
                    new String[1] { root }, false);

                for (int j = 0; j < Data.RegKeys.Count; j++)
                {
                    RegKeyEntry re = Data.RegKeys.RegKey((Int16)j);

                    switch (re.KeyDataType)
                    {
                        case 0x00010001: //DWORD
                            WriteParam(writer, re.KeyName, Convert.ToString(re.Value_DWORD), "datatype", "integer");
                            break;
                        case 0x00000000: //SZ 
                            if (re.Value_String.ToLower() == "%installdir%")
                                WriteParam(writer, re.KeyName, re.Value_String, "datatype", "string", "translation", "install");                               
                            else
                                WriteParam(writer, re.KeyName, re.Value_String, "datatype", "string");
                            break;
                        case 0x00001000: //MULTI_SZ :
                            String mzString = FileTools.MultiString2String(re.Value_Multi_String);                            
                            WriteParam(writer, re.KeyName, mzString, "datatype", "multiplestring");
                            break;
                        case 0x00000001:  //Binary
                            String binString = Convert.ToBase64String(re.Value_Binary);                            
                            WriteParam(writer, re.KeyName, binString, "datatype", "binary");
                            break;
                    }
            
                }

                writer.WriteEndElement();
            }

            writer.WriteEndElement();

        }

        public void Write(XmlWriter writer)
        {
            //writer.Formatting = Formatting.None;
            writer.WriteStartElement("wap-provisioningdoc");
            WriteSetupSection(writer);
            WriteFilesIconsSection(writer);            
            WriteRegistrySection(writer);
            writer.WriteEndElement();
        }

        public void Read(XmlTextReader reader)
        {
            reader.Read();
            if (reader.Name != "wap-provisioningdoc")
                throw new FormatException("Unrecognized tag in xml");            
            
            while (reader.Read())
            {
                if (reader.Name == "characteristic")
                	switch (reader.GetAttribute("type"))
                	{
                		case "Install": 		ReadSetupSection(reader); break;
                		case "FileOperation":	ReadFilesIconsSection(reader); break;
                	}
                	
            }
        }
    }
}
