using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace QuickZip.InstallBuilder
{
    internal abstract class Entry
    {
        public abstract Int16 MinLength();
        public abstract void DebugClass();
    }
    /// <summary>
    /// One item in [STRINGS] section.
    /// </summary>
    internal class StringEntry : Entry
    {
        public Int16 StringID, ValueLength;
        public String Value;

        public StringEntry()
        {

        }

        public StringEntry(Int16 ID, String s)
        {
            StringID = ID;
            Value = s;
            ValueLength = (Int16)(s.Length + 1);
        }

        public override Int16 MinLength()
        {
            ValueLength = (Int16)(Value.Length + 1);
            return (Int16)(2 + //StringID
                           2 + //Length
                           ValueLength);
        }

        public override void DebugClass()
        {
            Debug.WriteLine(String.Format(
                "  >{0,-10} = Value({1}), Length({2}/{3})"
                , StringID, Value, ValueLength, MinLength()));
        }
    }
    /// <summary>
    /// One item in [DIRS] section.
    /// </summary>
    internal class DirEntry : Entry
    {
        public Int16 DirID, ValueLength;
        public Int16[] Value;

        public DirEntry()
        {

        }

        public DirEntry(Int16 ID, Int16[] v)
        {
            DirID = ID;
            Value = v;
            ValueLength = (Int16)(Value.Length * 2);
        }

        public override Int16 MinLength()
        {
            ValueLength = (Int16)(Value.Length * 2);
            return (Int16)(2 + //DirID
                           2 + //Length
                           ValueLength);
        }

        public static String DecodeDirectory(String input, String defaultdir)
        {
            String value = input;
            if (input == defaultdir)
            {
                return @"{app}";
            }

            for (int i = 0; i <= 17; i++ )
            {
                PPC30 id = (PPC30)i;
                String FromString = String.Format("%CE{0}%", i);                
                String ToString = "{"+Convert.ToString(id)+"}";                
                value = value.Replace(FromString, ToString);
            }
            return value;
        }

        public static String DecodeDirectory2(String input, String defaultdir)
        {
            String value = input;

            for (int i = 0; i <= 17; i++)
            {
                PPC30 id = (PPC30)i;
                String FromString = String.Format("%CE{0}%", i);
                String ToString = "{" + Convert.ToString(id) + "}";
                value = value.Replace(FromString, ToString);
            }
            return value;
        }

        public static String DecodeDirectory(String input, String defaultdir, out PPC30 basedir)
        {
            String DecodedDir = DecodeDirectory(input, defaultdir);
            String[] SplittedDir = DecodedDir.Split(new Char[] { '\\' });
            basedir = PPC30.app;

            if (SplittedDir.Length > 1)
            {
                String dir = SplittedDir[0].ToLower();
                for (int i = 0; i <= 17; i++)
                {
                    String current = "{" + Convert.ToString((PPC30)(i)) + "}";
                    if (current == dir)
                    {
                        basedir = (PPC30)(i);
                    }
                }
            }

            String retDir = "";
            for (int i = 1; i < SplittedDir.Length; i++)
            {
                if (retDir == "")
                {
                    retDir = SplittedDir[i];
                }
                else
                {
                    retDir += @"\" + SplittedDir[i];
                }
            }
            return retDir;
        }

        public static String EncodeDirectory(String input, String defaultdir)
        {
            String value = input;
            if (defaultdir != "")
             value = input.Replace(defaultdir, "%CE0%");                       

            for (int i = 0; i <= 17; i++)
            {
                PPC30 id = (PPC30)i;
                String ToString = String.Format("%CE{0}%", i);
                
                String FromString = "{" + Convert.ToString(id) + "}";
                value = value.Replace(FromString, ToString);

                FromString = "{" + Convert.ToString(i) + "}";
                value = value.Replace(FromString, ToString);
            }
            return value;
        }
        public static String EncodeDirectory(String input)
        {
            return EncodeDirectory(input, "");
        }

        public override void DebugClass()
        {
            Debug.WriteLine(String.Format(
                "  >{0,-10} = Value({1}), Length({2})"
                , DirID, FileTools.Int16Array2String(Value), ValueLength));
        }        
    };

    /// <summary>
    /// One item in [FILES] section.
    /// </summary>
    internal class FileEntry : Entry
    {
        //0x00000010 DontOverwrite
        //0x00000400 UpdateOnly (Copy file only if it is replacing an existing file)
        //0x20000000 Dont update if existing newer
        //0x40000000 Ignore date while overwriting target file
        //0x00000001 Warn if file not copied
        //0x00000002 Do not allow user to skip file on error
		//No allow skip = 
		//No Overwrite = NoOverwrite
		//Replace Only = OverwriteIfExists
		//Copy newer = NoOverwriteIfNewer
		//Ignore date = NoDateCheck


		//<parm name="NoDateCheck" />
		//<parm name="NoSkip"/>
        //<parm name="SelfRegister"/>
        //<parm name="Shared"/>
        public enum FileEntryType : ulong
        {
            Normal              = 0x00000001,
            NoSkip              = 0x00000002,
            NoOverwrite         = 0x00000010,
            OverwriteIfExists   = 0x00000400,
            NoOverwriteIfNewer  = 0x20000000,
            NoDateCheck         = 0x40000000,

            SelfRegister        = 0x10000000, //ActiveX
            Shared              = 0x80000000
        }
        public FileEntryType Type
        {
            get
            {
                UInt32 count = Flags;
                if (IsShared)
                    count -= (UInt32)FileEntryType.Shared;
                if (IsSelfRegister)
                    count -= (UInt32)FileEntryType.SelfRegister;

                return (FileEntryType)count;
            }
            set
            {
                UInt32 count = (UInt32)value;
                if (IsShared)
                { count += (UInt32)FileEntryType.Shared; }
                if (IsSelfRegister)
                { count += (UInt32)FileEntryType.SelfRegister; }
                    
                Flags = (UInt32)count;
            }
        }
        public bool IsShared
        {
            get
            {
                UInt32 count = Flags;
                if (count >= (UInt32)FileEntryType.Shared)
                    return true;
                return false;
            }
            set
            {
                if (IsShared) Flags += (UInt32)FileEntryType.Shared;
            }
        }
        public bool IsSelfRegister
        {
            get
            {
                UInt32 count = Flags;
                if (count >= (UInt32)FileEntryType.Shared)
                    count -= (UInt32)FileEntryType.Shared;
                if (count >= (UInt32)FileEntryType.SelfRegister)
                    return true;
                return false;
            }
            set
            {
                if (value)
                    if (IsSelfRegister) Flags += (UInt32)FileEntryType.SelfRegister;
                    else
                        if (!(IsSelfRegister)) Flags -= (UInt32)FileEntryType.SelfRegister;
            }
        }
        public Int16 FileID, DirID, FileNumber;
        public UInt32 Flags;
        public Int16 ValueLength;
        public String Path;
        public String SourcePath;
        public String ShortPath
        {
            get
            {
                return Tools.GetDosFilename(Value, FileID);
            }
        }
        public String Value
        {
            get
            {
                return Tools.ExtractFileName(Path);
            }
            set
            {
                Path = value;
            }
        }


        public FileEntry()
        {

        }

        public FileEntry(Int16 ID, Int16 DID, Int16 number, UInt32 flags, String filename)
        {
            FileID = ID;
            DirID = DID;
            FileNumber = number;
            Flags = flags;
            Value = filename;            
            ValueLength = (Int16)(Value.Length + 1);
        }

        public FileEntry(Int16 ID, Int16 DID, Int16 number, UInt32 flags, String filename, String sourcefilename)
        {
            FileID = ID;
            DirID = DID;
            FileNumber = number;
            Flags = flags;
            Value = filename;
            SourcePath = sourcefilename;
            ValueLength = (Int16)(Value.Length + 1);
        }

        public override Int16 MinLength()
        {
            ValueLength = (Int16)(Value.Length+1);
            return (Int16)(2 + //FileID
                           2 + //DirID
                           2 + //FileNumber
                           4 + //Flags
                           2 + //Length
                           ValueLength);
        }

        public override void DebugClass()
        {
            Debug.WriteLine(String.Format(
                "  >{0,-10} = Value({1}), Number({2,3:000}), DirID({3}), Length({4}/{6}), Flags({5})"
                , FileID, Value, FileNumber, DirID, ValueLength, Flags,MinLength()));
        }
    };

    /// <summary>
    /// One item in [REGHIVES] section
    /// </summary>
    internal class RegHiveEntry : Entry
    {
        public Int16 HiveID;
        public RegistryRoot HiveRoot;
        public Int16 HiveUnknown; //usually 0 or 1
        public Int16 ValueLength;
        public Int16[] Value;

        public RegHiveEntry()
        {

        }

        public RegHiveEntry(Int16 ID, RegistryRoot root, Int16[] v)
        {
            HiveID = ID;
            HiveRoot = root;
            Value = v;
            ValueLength = (Int16)(v.Length * 2);
            HiveUnknown = 0;
            //TODO: Check value of HiveUnknown
        }

        public override Int16 MinLength()
        {
            return (Int16)(2 + //HiveID
                           2 + //HiveRoot
                           2 + //Unknown                           
                           2 + //Length
                           Value.Length*2);
        }

        public override void DebugClass()
        {
            Debug.WriteLine(String.Format(
                "  >{0,-10} = Root({1}), Value({2}), Unknown(0?)({3}), Length({4}/{5})"
                , HiveID, HiveRoot, FileTools.Int16Array2String(Value), HiveUnknown, ValueLength,MinLength()));
        }
    }

    /// <summary>
    /// One item in [REGKEYS] section
    /// </summary>
    internal class RegKeyEntry : Entry
    {
        public Int16 KeyID, HiveID;
        public Int16 KeyUnknown=1; //usually 0 or 1
        public Int32 KeyDataType;
        public Int16 ValueLength;
        public String KeyName;
        public Int32 Value_DWORD;
        public String Value_String;
        public String[] Value_Multi_String;
        public Byte[] Value_Binary;

        public RegKeyEntry()
        {

        }

        public RegKeyEntry(Int16 ID, Int16 HID, String keyname, Int32 value)
        {
            KeyID = ID;
            HiveID = HID;
            KeyName = keyname;
            Value_DWORD = value;
            KeyDataType = 0x00010001;
        }

        public RegKeyEntry(Int16 ID, Int16 HID, String keyname, String value)
        {
            KeyID = ID;
            HiveID = HID;
            KeyName = keyname;
            Value_String = value;
            KeyDataType = 0x00000000;
        }

        public RegKeyEntry(Int16 ID, Int16 HID, String keyname, String[] value)
        {
            KeyID = ID;
            HiveID = HID;
            KeyName = keyname;
            Value_Multi_String = value;
            KeyDataType = 0x00001000;
        }

        public RegKeyEntry(Int16 ID, Int16 HID, String keyname, Byte[] value)
        {
            KeyID = ID;
            HiveID = HID;
            KeyName = keyname;
            Value_Binary = value;
            KeyDataType = 0x00000001;
        }

        public override Int16 MinLength()
        {
            ValueLength = (Int16)(KeyName.Length + 1);
            switch (KeyDataType)
            {               
                case 0x00010001: //DWORD
                    ValueLength += 4;
                    break;
                case 0x00000000: //SZ 
                    ValueLength += (Int16)(Value_String.Length + 1);
                    break;
                case 0x00001000: //MULTI_SZ : 
                    for (Int16 i = 0; i < Value_Multi_String.Length; i++)
                    {
                        ValueLength += (Int16)(Value_Multi_String[i].Length + 1);
                    }                    
                    break;
                case 0x00000001:  //Binary
                    ValueLength += (Int16)(Value_Binary.Length);
                    break;
            }
            return (Int16)(2 + //KeyID
                           2 + //HiveID
                           2 + //Unknown
                           4 + //KeyDataType
                           2 + //Length
                           ValueLength);
        }

        public String DebugValue
        {
            get
            {                
                switch (KeyDataType)
                {
                    case 0x00010001: //DWORD
                        return Convert.ToString(Value_DWORD);                        
                    case 0x00000000: //SZ 
                        return Value_String;
                    case 0x00001000: //MULTI_SZ : 
                        return FileTools.StringArray2String(Value_Multi_String);
                    case 0x00000001:  //Binary                        
                        return FileTools.ByteArray2String(Value_Binary);

                    default: return "";
                }
            }
        }

        public override void DebugClass()
        {                        
            Debug.WriteLine(String.Format(
                "  >{0,-10} = HiveID({1}), Key({2}), DataType({3}), Value({4}), Unknown(1?)({5}), Length({6}/{7})"
                , KeyID, HiveID, KeyName, (RegistryDataType)KeyDataType, DebugValue, KeyUnknown, ValueLength,MinLength()));
        }
       
    }

    /// <summary>
    /// One item in LINKS Section
    /// </summary>
    internal class LinkEntry : Entry
    {
        public Int16 LinkID;
        public Int16 LinkUnknown=0;
        public PPC30 LinkBaseDir;
        public Int16 LinkTargetID;
        public LinkType LinkTargetType;
        public Int16 ValueLength;
        public Int16[] Value;

        public LinkEntry()
        {

        }

        public LinkEntry(Int16 ID, Int16 TID, LinkType linktargettype, PPC30 linkbasedir, Int16[] v)
        {
            LinkID = ID;
            LinkTargetID = TID;
            LinkTargetType = linktargettype;
            LinkBaseDir = linkbasedir;
            Value = v;
            ValueLength = (Int16)(v.Length * 2);
        }

        public override Int16 MinLength()
        {
            return (Int16)(2 + //LinkID
                           2 + //Unknown
                           2 + //BaseDir
                           2 + //File/Dir ID
                           2 + //LinkType
                           2 + //Specification Length
                           (Value.Length *2) //Specification
                           );
        }
      
        public override void DebugClass()
        {
            Debug.WriteLine(String.Format(
                "  >{0,-10} = BaseDir({1}), Target({2}), LinkType({3}), Specification({4}), Unknown(0?)({5}), Length({6}/{7})"
                , LinkID, (PPC30)LinkBaseDir, LinkTargetID, (LinkType)LinkTargetType,
                FileTools.Int16Array2String(Value), LinkUnknown,
                ValueLength,MinLength()));
        }
    }
    
    /// <summary>
    /// StringInfo for use in PROVIDER and UNSUPPORTED string in header
    /// </summary>
    internal class StringInfo
    {                
        public Int16 Offset, ValueLength;
        public String Value="";

        public Int16 MinLength()
        {
            if (Value == "")
            {
                ValueLength = 0;                 
            }
            else
            if (Value == null)
            {
                ValueLength = 0;
            }
            else
            {
                ValueLength = (Int16)(Value.Length + 1);                 
            }
            return ValueLength;
        }

        public void DebugClass(String name)
        {
            Debug.WriteLine(String.Format(
                "{0,-13} = Value({3}), Offset({1}), Length({2}/{4})"
                , name, Offset, ValueLength, Value, MinLength()));
        }        
    }

}
